/^with .*$/d
/^package .*$/d
/^private package .*$/d
/^end .*$/d
/^ .*$/d
/^\-.*$/d
/^$/d
/^%if yyerrok$/d
/^%if yyclearin$/d
/^%if private$/d
/^%if reentrant$/d
/^%if error$/d
/^%if debug$/d
/^%else$/d
/^%end$/d
/^%yydecl$/d
/^%yyinit$/d
/^%%[12345678] [a-z]*$/d
